use warnings;
use strict;

package XML::Compile::C14N;

use Log::Report 'xml-compile-c14n';

use XML::Compile::C14N::Util;

my %versions =
 ( '1.0' => {}
 , '1.1' => {}
 );

my @prefixes = (c14n => C14N_EXC_NS);

=chapter NAME
XML::Compile::C14N - XML Canonicalization

=chapter SYNOPSIS
 my $schema = XML::Compile::Cache->new(...);
 my $c14n   = XML::Compile::C14N->new(schema => $schema);
 
=chapter DESCRIPTION
XML canonicalization is used to enforce an explicit formatting style
on de XML documents. It is required to have a reproducable output when,
for instance, digital signatures gets applied to parts of the document.

WARNING: Although you may wish for more, this module currently only
helps to decode the C14N information, for instance used in digital
signatures. The actual process of canonicalization is not (yet)
implemented.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=option  version STRING
=default version '1.1'
Explicitly state which version C14N needs to be used.

=option  schema M<XML::Compile::Cache>
=default schema C<undef>
Add the C14N extension information to the provided schema.  If not used,
you have to call M<loadSchemas()> before compiling readers and writers.
=cut

sub new(@) { my $class = shift; (bless {}, $class)->init( {@_} ) }
sub init($)
{   my ($self, $args) = @_;

    my $version = $args->{version} || '1.1';
    trace "initializing v14n $version";

    $versions{$version}
        or error __x"unknown c14n version {v}, pick from {vs}"
             , v => $version, vs => [keys %versions];
    $self->{XCC_version} = $version;

    $self->loadSchemas($args->{schema})
        if $args->{schema};

    $self;
}

#-----------

=section Attributes

=method version
Returns the version number.
=method schema
=cut

sub version() {shift->{XCC_version}}
sub schema()  {shift->{XCC_schema}}

#-----------

=section Handling

=method loadSchemas SCHEMA
SCHEMA must extend M<XML::Compile::Cache>.
=cut

sub loadSchemas($)
{   my ($self, $schema) = @_;

    $schema->isa('XML::Compile::Cache')
        or error __x"loadSchemas() requires a XML::Compile::Cache object";
    $self->{XCC_schema} = $schema;

    my $version = $self->version;
    my $def = $versions{$version};

    $schema->prefixes(@prefixes);
    {   local $" = ',';
        $schema->addKeyRewrite("PREFIXED(@prefixes)");
    }

    (my $xsd = __FILE__) =~ s! \.pm$ !/exc-c14n.xsd!x;
    trace "loading c14n for $version";

    $schema->importDefinitions($xsd);
    $self;
}

=section SEE ALSO
=over 4
=item Canonical XML Version 1.0
F<http://www.w3.org/TR/2001/REC-xml-c14n-20010315>, 15 March 2001

=item Canonical XML Version 1.1
F<http://www.w3.org/TR/2008/REC-xml-c14n11-20080502/>, 2 May 2008

=back
=cut

1;
